##############################################################################
# Copyright 2019 IBM Corp.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##############################################################################

import os
from importlib import import_module

_PATH = os.path.abspath(os.path.dirname(__file__))
RESOURCES = \
    [resource for resource in os.listdir(_PATH) if os.path.isdir(
        os.path.join(_PATH, resource)
        )
     ]
RESOURCE_NAME_CLASS_MAP = {}

for re in RESOURCES:
    if re != '__pycache__':
        RESOURCE_NAME_CLASS_MAP.update(
            import_module('{0}.{1}'.format(__name__,
                                           re)).RESOURCE_NAME_CLASS_MAP
        )
