##############################################################################
# Copyright 2019 IBM Corp.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##############################################################################

ALL = {
  "server": {
    "status": "ok",
    "code": "",
    "message": "Operation done successfully."
  },
  "counts": {
    "data_counts": 12,
    "total_counts": 12
  },
  "data": {
    "marrays": [
      {
        "id": "MA1",
        "link": {
          "rel": "self",
          "href": "https://localhost:8088/api/v1/marrays/MA1"
        },
        "disk_class": "enterprise",
        "pool": {
          "id": "P0",
          "link": {
            "rel": "self",
            "href": "https://localhost:8088/api/v1/pools/P0"
          }
        }
      },
      {
        "id": "MA2",
        "link": {
          "rel": "self",
          "href": "https://localhost:8088/api/v1/marrays/MA2"
        },
        "disk_class": "enterprise",
        "pool": {
          "id": "P1",
          "link": {
            "rel": "self",
            "href": "https://localhost:8088/api/v1/pools/P1"
          }
        }
      },
      {
        "id": "MA3",
        "link": {
          "rel": "self",
          "href": "https://localhost:8088/api/v1/marrays/MA3"
        },
        "disk_class": "enterprise",
        "pool": {
          "id": "P2",
          "link": {
            "rel": "self",
            "href": "https://localhost:8088/api/v1/pools/P2"
          }
        }
      },
      {
        "id": "MA4",
        "link": {
          "rel": "self",
          "href": "https://localhost:8088/api/v1/marrays/MA4"
        },
        "disk_class": "enterprise",
        "pool": {
          "id": "P3",
          "link": {
            "rel": "self",
            "href": "https://localhost:8088/api/v1/pools/P3"
          }
        }
      },
      {
        "id": "MA5",
        "link": {
          "rel": "self",
          "href": "https://localhost:8088/api/v1/marrays/MA5"
        },
        "disk_class": "enterprise",
        "pool": {
          "id": "P4",
          "link": {
            "rel": "self",
            "href": "https://localhost:8088/api/v1/pools/P4"
          }
        }
      },
      {
        "id": "MA6",
        "link": {
          "rel": "self",
          "href": "https://localhost:8088/api/v1/marrays/MA6"
        },
        "disk_class": "enterprise",
        "pool": {
          "id": "P5",
          "link": {
            "rel": "self",
            "href": "https://localhost:8088/api/v1/pools/P5"
          }
        }
      },
      {
        "id": "MA7",
        "link": {
          "rel": "self",
          "href": "https://localhost:8088/api/v1/marrays/MA7"
        },
        "disk_class": "enterprise",
        "pool": {
          "id": "P6",
          "link": {
            "rel": "self",
            "href": "https://localhost:8088/api/v1/pools/P6"
          }
        }
      },
      {
        "id": "MA8",
        "link": {
          "rel": "self",
          "href": "https://localhost:8088/api/v1/marrays/MA8"
        },
        "disk_class": "enterprise",
        "pool": {
          "id": "P7",
          "link": {
            "rel": "self",
            "href": "https://localhost:8088/api/v1/pools/P7"
          }
        }
      },
      {
        "id": "MA9",
        "link": {
          "rel": "self",
          "href": "https://localhost:8088/api/v1/marrays/MA9"
        },
        "disk_class": "enterprise",
        "pool": {
          "id": "P8",
          "link": {
            "rel": "self",
            "href": "https://localhost:8088/api/v1/pools/P8"
          }
        }
      },
      {
        "id": "MA10",
        "link": {
          "rel": "self",
          "href": "https://localhost:8088/api/v1/marrays/MA10"
        },
        "disk_class": "enterprise",
        "pool": {
          "id": "P9",
          "link": {
            "rel": "self",
            "href": "https://localhost:8088/api/v1/pools/P9"
          }
        }
      },
      {
        "id": "MA11",
        "link": {
          "rel": "self",
          "href": "https://localhost:8088/api/v1/marrays/MA11"
        },
        "disk_class": "enterprise",
        "pool": {
          "id": "P10",
          "link": {
            "rel": "self",
            "href": "https://localhost:8088/api/v1/pools/P10"
          }
        }
      },
      {
        "id": "MA12",
        "link": {
          "rel": "self",
          "href": "https://localhost:8088/api/v1/marrays/MA12"
        },
        "disk_class": "enterprise",
        "pool": {
          "id": "P11",
          "link": {
            "rel": "self",
            "href": "https://localhost:8088/api/v1/pools/P11"
          }
        }
      }
    ]
  }
}


ONE = {
  "server": {
    "status": "ok",
    "code": "",
    "message": "Operation done successfully."
  },
  "counts": {
    "data_counts": 12,
    "total_counts": 12
  },
  "data": {
    "marrays": [
      {
        "id": "MA1",
        "link": {
          "rel": "self",
          "href": "https://localhost:8088/api/v1/marrays/MA1"
        },
        "disk_class": "enterprise",
        "pool": {
          "id": "P0",
          "link": {
            "rel": "self",
            "href": "https://localhost:8088/api/v1/pools/P0"
          }
        }
      },
           ]
        }
       }
