##############################################################################
# Copyright 2019 IBM Corp.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##############################################################################

ALL = {
    'data': {
        'default': [
            {
                'id': 'a',
                'link': {
                    'rel': 'self',
                    'href': '/default/a'
                },
            },
            {
                'id': 'b',
                'link': {
                    'rel': 'self',
                    'href': '/default/b'
                },
            }
        ]
    }
}

ONE = {
    'data': {
        'default': [
            {'name': 'name_a',
             'id': 'a',
             'link': {
                 'rel': 'self',
                 'href': '/default/a'
             },
             }
        ]
    }
}
