##############################################################################
# Copyright 2019 IBM Corp.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##############################################################################

ALL = {
  "server": {
    "status": "ok",
    "code": "CMUC00183I",
    "message": "Operation done successfully."
  },
  "counts": {
    "data_counts": 1,
    "total_counts": 1
  },
  "data": {
    "systems": [
      {
        "id": "2107-75DHZ81",
        "name": "mtc032h",
        "state": "online",
        "release": "7.4",
        "bundle": "87.40.47.0",
        "MTM": "2421-961",
        "sn": "75DHZ81",
        "wwnn": "5005076306FFD2F0",
        "cap": "440659",
        "capalloc": "304361",
        "capavail": "136810",
        "capraw": "73282879488"
      }
    ]
  }
}

ONE = ALL
