##############################################################################
# Copyright 2019 IBM Corp.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##############################################################################

DS8K_TOKEN = 'tokens'
DS8K_SYSTEM = 'systems'
DS8K_NODE = 'nodes'
DS8K_MARRAY = 'marrays'
DS8K_USER = 'users'
DS8K_ENCRYPTION_GROUP = 'encryption_groups'
DS8K_IOENCLOSURE = 'io_enclosures'
DS8K_POOL = 'pools'
DS8K_TSEREP = 'tserep'
DS8K_ESEREP = 'eserep'
DS8K_LSS = 'lss'
DS8K_VOLUME = 'volumes'
DS8K_SE = 'SE'
DS8K_VOLUME_GROUP = 'volgrps'
DS8K_HOST_PORT = 'host_ports'
DS8K_HOST = 'hosts'
DS8K_IOPORT = 'ioports'
DS8K_VOLMAP = 'mappings'
DS8K_EVENT = 'events'
DS8K_VOLUME_TYPE_FB = 'fb'
DS8K_VOLUME_TYPE_CKD = 'ckd'
DS8K_VOLUME_TYPES = (DS8K_VOLUME_TYPE_FB, DS8K_VOLUME_TYPE_CKD)
DS8K_FLASHCOPY = 'flashcopy'
DS8K_PPRC = 'pprc'
DS8K_CAPTYPE_GIB = 'gib'
DS8K_CAPTYPE_BYTE = 'bytes'
DS8K_CAPTYPE_CYL = 'cyl'
DS8K_CAPTYPE_MOD1 = 'mod1'
DS8K_CAPTYPES = (DS8K_CAPTYPE_GIB,
                 DS8K_CAPTYPE_BYTE,
                 DS8K_CAPTYPE_CYL,
                 DS8K_CAPTYPE_MOD1
                 )
DS8K_TP_NONE = 'none'
DS8K_TP_ESE = 'ese'
DS8K_TP_TSE = 'tse'
DE8K_TPS = (DS8K_TP_NONE, DS8K_TP_ESE, DS8K_TP_TSE)
