##############################################################################
# Copyright 2019 IBM Corp.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##############################################################################

ALL = {
  "server": {
    "status": "ok",
    "code": "",
    "message": "Operation done successfully."
  },
  "counts": {
    "data_counts": 25,
    "total_counts": 25
  },
  "data": {
    "volumes": [
      {
        "id": "0000",
        "link": {
          "rel": "self",
          "href": "https://localhost:8088/api/v1/volumes/0000"
        },
        "name": "RegrFBVol1",
        "state": "normal",
        "cap": "1073741824",
        "stgtype": "FB",
        "VOLSER": "",
        "lss": {
          "id": "00",
          "link": {
            "rel": "self",
            "href": "https://localhost:8088/api/v1/lss/00"
          }
        },
        "allocmethod": "rotatevols",
        "tp": "none",
        "MTM": "2107-900",
        "datatype": "FB 512",
        "easytier": "unknown",
        "tieralloc": [
          {
            "tier": "ENT",
            "allocated": "1073741824"
          }
        ],
        "pool": {
          "id": "P2",
          "link": {
            "rel": "self",
            "href": "https://localhost:8088/api/v1/pools/P2"
          }
        }
      },
      {
        "id": "0001",
        "link": {
          "rel": "self",
          "href": "https://localhost:8088/api/v1/volumes/0001"
        },
        "name": "RegrFBVol1",
        "state": "normal",
        "cap": "1073741824",
        "stgtype": "FB",
        "VOLSER": "",
        "lss": {
          "id": "00",
          "link": {
            "rel": "self",
            "href": "https://localhost:8088/api/v1/lss/00"
          }
        },
        "allocmethod": "rotatevols",
        "tp": "none",
        "MTM": "2107-900",
        "datatype": "FB 512",
        "easytier": "unknown",
        "tieralloc": [
          {
            "tier": "ENT",
            "allocated": "1073741824"
          }
        ],
        "pool": {
          "id": "P2",
          "link": {
            "rel": "self",
            "href": "https://localhost:8088/api/v1/pools/P2"
          }
        }
      },
      {
        "id": "0002",
        "link": {
          "rel": "self",
          "href": "https://localhost:8088/api/v1/volumes/0002"
        },
        "name": "RegrFBVol1",
        "state": "normal",
        "cap": "1073741824",
        "stgtype": "FB",
        "VOLSER": "",
        "lss": {
          "id": "00",
          "link": {
            "rel": "self",
            "href": "https://localhost:8088/api/v1/lss/00"
          }
        },
        "allocmethod": "rotatevols",
        "tp": "none",
        "MTM": "2107-900",
        "datatype": "FB 512",
        "easytier": "unknown",
        "tieralloc": [
          {
            "tier": "ENT",
            "allocated": "1073741824"
          }
        ],
        "pool": {
          "id": "P2",
          "link": {
            "rel": "self",
            "href": "https://localhost:8088/api/v1/pools/P2"
          }
        }
      },
      {
        "id": "0003",
        "link": {
          "rel": "self",
          "href": "https://localhost:8088/api/v1/volumes/0003"
        },
        "name": "RegrFBVol1",
        "state": "normal",
        "cap": "1073741824",
        "stgtype": "FB",
        "VOLSER": "",
        "lss": {
          "id": "00",
          "link": {
            "rel": "self",
            "href": "https://localhost:8088/api/v1/lss/00"
          }
        },
        "allocmethod": "rotatevols",
        "tp": "none",
        "MTM": "2107-900",
        "datatype": "FB 512",
        "easytier": "unknown",
        "tieralloc": [
          {
            "tier": "ENT",
            "allocated": "1073741824"
          }
        ],
        "pool": {
          "id": "P2",
          "link": {
            "rel": "self",
            "href": "https://localhost:8088/api/v1/pools/P2"
          }
        }
      },
      {
        "id": "0004",
        "link": {
          "rel": "self",
          "href": "https://localhost:8088/api/v1/volumes/0004"
        },
        "name": "RegrFBVol1",
        "state": "normal",
        "cap": "1073741824",
        "stgtype": "FB",
        "VOLSER": "",
        "lss": {
          "id": "00",
          "link": {
            "rel": "self",
            "href": "https://localhost:8088/api/v1/lss/00"
          }
        },
        "allocmethod": "rotatevols",
        "tp": "none",
        "MTM": "2107-900",
        "datatype": "FB 512",
        "easytier": "unknown",
        "tieralloc": [
          {
            "tier": "ENT",
            "allocated": "1073741824"
          }
        ],
        "pool": {
          "id": "P2",
          "link": {
            "rel": "self",
            "href": "https://localhost:8088/api/v1/pools/P2"
          }
        }
      },
      {
        "id": "0018",
        "link": {
          "rel": "self",
          "href": "https://localhost:8088/api/v1/volumes/0018"
        },
        "name": "singlevol2",
        "state": "normal",
        "cap": "1000013824",
        "stgtype": "FB",
        "VOLSER": "",
        "lss": {
          "id": "00",
          "link": {
            "rel": "self",
            "href": "https://localhost:8088/api/v1/lss/00"
          }
        },
        "allocmethod": "rotateexts",
        "tp": "none",
        "MTM": "2107-900",
        "datatype": "FB 512",
        "easytier": "unknown",
        "tieralloc": [
          {
            "tier": "ENT",
            "allocated": "1073741824"
          }
        ],
        "pool": {
          "id": "P2",
          "link": {
            "rel": "self",
            "href": "https://localhost:8088/api/v1/pools/P2"
          }
        }
      }
    ]
  }
}

ONE = {
  "server": {
    "status": "ok",
    "code": "CMUC00183I",
    "message": "Operation done successfully."
  },
  "counts": {
    "data_counts": 1,
    "total_counts": 1
  },
  "data": {
    "volumes": [
      {
        "id": "0000",
        "link": {
          "rel": "self",
          "href": "https://localhost:8088/api/v1/volumes/0000"
        },
        "name": "RegrFBVol1",
        "state": "normal",
        "cap": "1073741824",
        "stgtype": "FB",
        "VOLSER": "",
        "lss": {
          "id": "00",
          "link": {
            "rel": "self",
            "href": "https://localhost:8088/api/v1/lss/00"
          }
        },
        "allocmethod": "rotatevols",
        "tp": "none",
        "MTM": "2107-900",
        "datatype": "FB 512",
        "easytier": "unknown",
        "tieralloc": [
          {
            "tier": "ENT",
            "allocated": "1073741824"
          }
        ],
        "pool": {
          "id": "P2",
          "link": {
            "rel": "self",
            "href": "https://localhost:8088/api/v1/pools/P2"
          }
        }
      }
    ]
  }
}
